import numpy as np
import matplotlib.pyplot as plt
from math import floor

# plot trajectory of genetic algorithm
def plot_ga_traj(fname, init_population_size, n_children_per_gen, plot_color, shift=0, plot_label=None):

    # plot children generated for each generation
    traj = np.loadtxt(fname)
    n_samples = len(traj)
    for i_sample in range(n_samples):
        if i_sample < 10:
            gen = 0
        else:
            gen = floor((i_sample - 10) / n_children_per_gen)
        plt.plot(gen+shift, 1./traj[i_sample], 'o', color=plot_color, alpha=0.5, label=r''+plot_label)

    # plot running minimum
    min_found = float('Inf')
    plot_min = False
    gen_min = 0
    for i_sample in range(n_samples):
        if i_sample < 10:
            gen = 0
        else:
            gen = floor((i_sample - 10) / n_children_per_gen)
        if 1./traj[i_sample] < min_found:
            if plot_min:
                plt.plot([gen_min+shift, gen+shift], [min_found, min_found], color='k', linewidth=2)
                plt.plot([gen+shift, gen+shift], [min_found, 1./traj[i_sample]], color='k', linewidth=2)
            plot_min = True
            min_found = 1./traj[i_sample]
            gen_min = gen
    plt.plot([gen_min+shift, gen+shift], [min_found, min_found], color='k', linewidth=2)

if __name__ == '__main__':

    plt.subplot(1, 3, 1)
    plot_ga_traj('run10/test_GA_fitnesses.txt', 10, 2, 'b', plot_label='Fixed mutation rate, $n_{steps}=2000$')
    plt.text(10, 1.75, r'a) $\gamma$ fixed, $n_{steps}=2000$', fontsize=8)
    plt.xlim([0, 200])
    plt.ylim([0, 2])
    plt.ylabel(r'$S_{rel}$')
    plt.xlabel('Generation')

    plt.subplot(1, 3, 2)
    plot_ga_traj('run11/test_GA_fitnesses.txt', 10, 2, 'r', plot_label='Decaying mutation rate, $n_{steps}=2000$')
    plt.text(10, 1.75, r'b) $\gamma$ decay, $n_{steps}=2000$', fontsize=8)
    plt.xlim([0, 200])
    plt.ylim([0, 2])
    plt.xticks([100, 200], ['100', '200'])
    plt.yticks([], [])

    plt.subplot(1, 3, 3)
    plot_ga_traj('run12/test_GA_fitnesses.txt', 10, 2, 'g', plot_label='Decaying mutation rate, $n_{steps}=5000$')
    plt.text(10, 1.75, r'c) $\gamma$ decay, $n_{steps}=5000$', fontsize=8)
    plt.xlim([0, 200])
    plt.ylim([0, 2])
    plt.xticks([100, 200], ['100', '200'])
    plt.yticks([], [])

    plt.subplots_adjust(wspace=0, hspace=0)

    plt.savefig('ga_traj.png', dpi=200, bbox_inches='tight')
