import numpy as np
import matplotlib.pyplot as plt

from plot_ree import plot_pr

# plot the results from the genetic algorithm

def plot_pr_from_file(fname, plot_label=''):
    data = np.loadtxt(fname)
    p = plt.plot(data[:, 0], data[:, 1], '--', label=plot_label)
    return p

def plot_opt_pr(folder, plot_label=''):
    fitnesses = np.loadtxt(folder + '/test_GA_fitnesses.txt')
    i_max = np.argmax(fitnesses)
    print(folder, i_max)
    p = plot_pr_from_file(folder + '/opt_reedist_' + str(i_max) + '.txt', plot_label=plot_label)
    return p

plt.subplot(2, 2, 1)
plot_pr('ree_seq11111n1n1n1n1n111111_T1.0_run1.txt', plot_mean=False, plot_label='Target')
p = plot_opt_pr('run12', plot_label='Optimized')
plot_pr('ree_seq1111n1n1n1n1n1111111_T1.0.txt', plot_mean=False, plot_label='Refined', plot_format='^:')
plt.text(0.5, 0.45, 'a) hydrophobic ends', fontsize=8)
plt.legend(edgecolor='None', fontsize=8)
plt.xlim([0, 10])
plt.ylim([0, 0.5])
plt.xticks([], [])
plt.yticks([0.25, 0.5], ['0.25', '0.50'])

plt.subplot(2, 2, 2)
plot_pr('ree_seq11111111n1n1n1n1n1n1n1_T1.0_run1.txt', plot_mean=False, plot_label='Target')
p = plot_opt_pr('run14_halfhydrophobic')
plot_pr('run14_halfhydrophobic/ree_seq1111111n11n1n1n11n1n1_T1.0.txt', plot_mean=False, plot_label='', plot_format='^:')
plt.text(0.5, 0.45, 'b) diblock copolymer', fontsize=8)
plt.xlim([0, 10])
plt.ylim([0, 0.5])
plt.xticks([], [])
plt.yticks([], [])

plt.subplot(2, 2, 3)
plot_pr('ree_seq1n11n11n11n11n11n11n11_T1.0_run1.txt', plot_mean=False, plot_label='Target')
p = plot_opt_pr('run15_alternating')
plot_pr('ree_seq1n11n11111n1n1n11n1n11_T1.0.txt', plot_mean=False, plot_format='^:')
plt.text(0.5, 0.45, 'c) alternating', fontsize=8)
plt.xlim([0, 10])
plt.ylim([0, 0.5])
plt.xlabel(r'$R_{ee}$')
plt.ylabel(r'$P(R_{ee})$')
plt.xticks([0, 2, 4, 6, 8, 10], ['0', '2', '4', '6', '8', '10'])
plt.yticks([0.00, 0.25, 0.5], ['0.00', '0.25', '0.50'])

plt.subplot(2, 2, 4)
plot_pr('ree_fictitious.txt', plot_mean=False, plot_label='Target')
p = plot_opt_pr('run13')
plot_pr('ree_seq11n11111n1n1n111111_T1.0.txt', plot_mean=False, plot_format='^:')
plt.text(0.5, 0.45, 'd) fictitious', fontsize=8)
plt.xlim([0, 10])
plt.ylim([0, 0.5])
plt.xticks([2, 4, 6, 8, 10], ['2', '4', '6', '8', '10'])
plt.yticks([], [])

plt.subplots_adjust(wspace=0, hspace=0)

plt.savefig('ga_results.png', dpi=200, bbox_inches='tight')


