! calculate full energy
subroutine CalcEnergy(Pos, Types, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NAtom-1) :: Types
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), dimension(Dim) :: rij, Posi, r1, r2
    real(8) :: d2, id2, id6, id12, rc2, CoeffNB
    real(8), parameter :: kang = 0.25, knb = 0.125
    integer :: i, j

    PEnergy = 0.

    rc2 = rc * rc

    do i = 0, NAtom - 1
       Posi = Pos(i,:)
       if (i < (NAtom-2)) then
          ! angle interaction
          r1 = Pos(i+1,:)-Posi
          r2 = Pos(i+2,:)-Pos(i+1,:)
          PEnergy = PEnergy + kang * (1 - (sum(r1 * r2) / dsqrt(sum(r1 * r1) * sum(r2 * r2))))
       endif

       do j = i + 2, NAtom - 1
          rij = Pos(j,:) - Posi
          d2 = sum(rij * rij)
          if (d2 <= rc2) then
             id2 = 1. / d2
             id6 = id2 * id2 * id2
             id12 = id6 * id6
             CoeffNB = knb * (1 + Types(i) + Types(j) + 5 * Types(i) * Types(j))
             PEnergy = PEnergy + 4. * (id12 - CoeffNB * id6)
          endif
       enddo
    enddo
end subroutine

! calculate non-bonded energy for single bead
subroutine CalcExtEnergySingleParticle(Pos, Types, rc, PEnergy, Dim, NAtom, iAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom, iAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NAtom-1) :: Types
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12, rc2, CoeffNB
    real(8), parameter :: kang = 0.25, knb = 0.125
    integer :: j

    PEnergy = 0.

    rc2 = rc * rc

    Posi = Pos(iAtom,:)
    do j = 0, NAtom - 1
       if (j == iAtom) then
          cycle
       endif
       if (j == iAtom - 1) then
          cycle
       endif
       if (j == iAtom + 1) then
          cycle
       endif

       rij = Pos(j,:) - Posi
       d2 = sum(rij * rij)
       if (d2 <= rc2) then
          id2 = 1. / d2
          id6 = id2 * id2 * id2
          id12 = id6 * id6
          CoeffNB = knb * (1 + Types(iAtom) + Types(j) + 5 * Types(iAtom) * Types(j))
          PEnergy = PEnergy + 4. * (id12 - CoeffNB * id6)
       endif
    enddo
end subroutine

! calculate full energy for single bead
subroutine CalcEnergySingleParticle(Pos, Types, rc, PEnergy, Dim, NAtom, iAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom, iAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NAtom-1) :: Types
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), dimension(Dim) :: rij, Posi, r1, r2
    real(8) :: d2, id2, id6, id12, rc2, CoeffNB
    real(8), parameter :: kang = 0.25, knb = 0.125
    integer :: j

    PEnergy = 0.

    rc2 = rc * rc

    Posi = Pos(iAtom,:)

    ! angle interaction with iAtom on left
    if (iAtom < (NAtom - 2)) then
       r1 = Pos(iAtom+2,:)-Pos(iAtom+1,:)
       r2 = Pos(iAtom+1,:)-Posi
       PEnergy = PEnergy + kang * (1 - (sum(r1 * r2) / dsqrt(sum(r1 * r1) * sum(r2 * r2))))
    endif       

    ! angle interaction with iAtom in center
    if (iAtom > 0 .and. iAtom < (NAtom - 1)) then
       r1 = Pos(iAtom+1,:)-Posi
       r2 = Posi-Pos(iAtom-1,:)
       PEnergy = PEnergy + kang * (1 - (sum(r1 * r2) / dsqrt(sum(r1 * r1) * sum(r2 * r2))))
    endif

    ! angle interaction with iAtom on right
    if (iAtom > 1) then
       r1 = Posi-Pos(iAtom-1,:)
       r2 = Pos(iAtom-1,:)-Pos(iAtom-2,:)
       PEnergy = PEnergy + kang * (1 - (sum(r1 * r2) / dsqrt(sum(r1 * r1) * sum(r2 * r2))))
    endif
       
    do j = 0, NAtom - 1
       if (j == iAtom) then
          cycle
       endif
       if (j == iAtom - 1) then
          cycle
       endif
       if (j == iAtom + 1) then
          cycle
       endif

       rij = Pos(j,:) - Posi
       d2 = sum(rij * rij)
       if (d2 <= rc2) then
          id2 = 1. / d2
          id6 = id2 * id2 * id2
          id12 = id6 * id6
          CoeffNB = knb * (1 + Types(iAtom) + Types(j) + 5 * Types(iAtom) * Types(j))
          PEnergy = PEnergy + 4. * (id12 - CoeffNB * id6)
       endif
    enddo
end subroutine

! calculate fluctuations needed for derivative of energy
subroutine CalcAlchemicalFluctuations(Pos, Types, rc, Flucs, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NAtom-1) :: Types
    real(8), intent(in) :: rc
    real(8), intent(inout), dimension(0:NAtom-1) :: Flucs
!f2py intent(in, out) :: Flucs
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, rc2
    integer :: i, j

    Flucs = 0.

    rc2 = rc * rc

    do i = 0, NAtom - 1
       Posi = Pos(i, :)
       do j = 0, NAtom - 1
          if (i == j) then 
             cycle
          endif
          if (i == j - 1) then
             cycle
          endif
          if (i == j + 1) then
             cycle
          endif
          rij = Pos(j, :) - Posi
          d2 = sum(rij * rij)
          if (d2 <= rc2) then
             id2 = 1. / d2
             id6 = id2 * id2 * id2
             Flucs(i) = Flucs(i) + id6 * (1 + 5 * Types(j))
          endif
       enddo
    enddo
end subroutine
